DrumCoord = LibStub("AceAddon-3.0"):NewAddon("DrumCoord", "AceComm-3.0", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")


local toboolean = function(str)
	if (str == "true") then
		return true
	elseif (str == "false") then
		return false
	else
		return nil
	end
end

local function DEC_HEX(IN)
    local B,K,OUT,I,D=16,"0123456789ABCDEF","",0
	if IN == 0 then return IN end
    while IN>0 do
        I=I+1
        IN,D=math.floor(IN/B),math.mod(IN,B)+1
        OUT=string.sub(K,D,D)..OUT
    end
    return OUT
end

local function round(num, idp)
	local mult = 10 ^ (idp or 0)
	return math.floor(num * mult + 0.5) / mult
end

local defaults = {
    profile = {
        showInChat = false,
		showInParty = true,
        showAsWarning = true,
		showWhileSolo = true,
		showWhileUnusable = true,
		showWhileNoCombat = false,
		prepWarning = 3,
		showOnCombat = true,
		showDrumsDrpping = true,
		showCooldownEnded = true,
		tempUsers = {},
		messageColor = {r=1,g=1,b=1,a=1},
		colorstr = "ffffffff",
    },
}

local optionsFrame

local options = {
    name = "DrumCoord",
    handler = DrumCoord,
    type = 'group',
    args = {
		display = {
			type = 'group',
			name = "Display Options",
			args = {
				label = {
					type = "header",
					name = "Show announces...",
					order = 0,
				},
				chat = {
		            type = "toggle",
		            name = "...in the chat window.",
		            desc = "Toggles the display of local (self-only) messages in the chat window.\n\nGets spammy if both chat and party are enabled.",
		            get = "IsShowInChat",
		            set = "ToggleShowInChat",
					order = 1,
				},
				party = {
		            type = "toggle",
		            name = "...in party chat.",
		            desc = "Toggles the printing of messages in party chat.\n\nGets spammy if both chat and party are enabled.",
		            get = "IsShowInParty",
		            set = "ToggleShowInParty",
					order = 2,
				},
				warning = {
		            type = "toggle",
		            name = "...as warning.",
		            desc = "Toggles the display of messages on the screen as raid warning.",
		            get = "IsShowAsWarning",
		            set = "ToggleShowAsWarning",
					order = 3,
				},
				whileSolo = {
					type = "toggle",
					name = "...while solo.",
					desc = "Toggles the display of messages while ungrouped.",
					get = "IsShowWhileSolo",
					set = "ToggleShowWhileSolo",
					order = 4,
				},
				whileUnusable = {
					type = "toggle",
					name = "...while unusable.",
					desc = "Toggle the display of messages even if you are not a sufficiently skilled Leatherworker.",
					get = "IsShowWhileUnusable",
					set = "ToggleShowWhileUnusable",
					order = 5,
				},
				whileNoCombat = {
					type = "toggle",
					name = "...while not fighting.",
					desc = "Toggle the display of messages if you are not in combat.",
					get = "IsShowWhileNoCombat",
					set = "ToggleShowWhileNoCombat",
					order = 6,
				},
				prepWarning = {
					type = "range",
					name = "Prep Warning",
					desc = "How far ahead of time DrumCoord should warn you that a drum effect will drop.\nDOES NOT AFFECT OTHER USERS.",
					min = 0,
					max = 10,
					bigStep = 0.5,
					get = "GetPrepWarning",
					set = "SetPrepWarning",
				},
			},
        },
		msgconfig = {
			type = 'group',
			name = "Messages",
			args = {
				onCombat = {
					type = "toggle",
					name = "First drums!",
					desc = "Toggles the announcement of an available drummer when you enter combat.",
					get = "IsShowOnCombat",
					set = "ToggleShowOnCombat",
					order = 1,
				},
				drumsDropping = {
					type = "toggle",
					name = "Drums dropping...",
					desc = "Toggles the \"<Drummer>'s drums dropping in <time>...\" message.",
					get = "IsShowDrumsDropping",
					set = "ToggleShowDrumsDropping",
					order = 2,
				},
				cooldownEnded = {
					type = "toggle",
					name = "Cooldown ended.",
					desc = "Toggles the \"<Drummer>'s cooldown ended.\" message.",
					get = "IsShowCooldownEnded",
					set = "ToggleShowCooldownEnded",
					order = 3,
				},
				messageColor = {
					type = "color",
					name = "Message Color",
					desc = "Color of the messages that will be shown.\n\nDoes not apply to messages sent in party chat.",
					get = "GetMessageColor",
					set = "SetMessageColor",
					order = 4,
				},
			},
		},
    },
}

function DrumCoord:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("DrumCoordDB", defaults, "Default")
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("DrumCoord", options)
	optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DrumCoord", "DrumCoord")
	self:RegisterChatCommand("drumcoord","ChatCommand")
	self:RegisterChatCommand("dc","ChatCommand")
end

function DrumCoord:OnEnable()
	self:RegisterEvent("PARTY_MEMBERS_CHANGED",self.PARTY_UPDATE) -- We need to keep track of the group and the leatherworkers in it
	self:RegisterEvent("BAG_UPDATE") -- We need to watch the user's drums and make sure he has some
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED") -- We need to watch for drum use, obviously
	self:RegisterEvent("CHAT_MSG_SKILL") -- We may need to update canUse if the user reaches 350 LW
	self:RegisterEvent("PLAYER_ENTERING_WORLD",self.PARTY_UPDATE)
	self:RegisterEvent("PLAYER_ALIVE",self.PARTY_UPDATE)
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterComm("DrumCoord", "ReceiveMessage")

--	self.lowDrumsWarning = 10
	--self.party = {}
	--self.numDrums = 0
	self.isAlive = UnitIsConnected("player") == 1
	self.inCombat = false
	
	self:UpdateCanDrum()
	-- self.watchTimer = nil
	-- self.warningTimer = nil
	-- self.nextTimer = nil
end

function DrumCoord:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToFrame(optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(DrumCoord, "dc", "DrumCoord", input)
    end
end

function DrumCoord:UpdateCanUse()
	if self.canUse then return end
	for i=1,GetNumSkillLines() do
		local name,_,expanded,rank = GetSkillLineInfo(i)
		if (expanded == nil) then
			ExpandSkillHeader(i)
		end
		if ((name == "Leatherworking") and (rank >= 350)) then
			self.canUse = true
			return
		end
	end
	self.canUse = false
end

function DrumCoord:UpdateCanDrum()
	self:UpdateCanUse()
	self.numDrums = GetItemCount("Drums of Battle")
	local canDrum = self.canUse and self.numDrums>0
	if canDrum ~= self.canDrum then
		self.canDrum = canDrum
		self:SendCommMessage("DrumCoord", tostring(self.canDrum), "PARTY")
	end
end

function DrumCoord:PLAYER_REGEN_DISABLED()
	self.inCombat = true
	if self:GetNextAvailDrummer() and self.db.profile.showOnCombat then
		self:Announce(self:GetNextAvailDrummer().." has first drums.")
	end
--	self:RegisterWatchTimer()
end

function DrumCoord:PLAYER_REGEN_ENABLED()
	self.inCombat = false
end

function DrumCoord:CHAT_MSG_SKILL()
	self:UpdateCanUse()
end

function DrumCoord:PARTY_UPDATE()
	local self = DrumCoord
	self.party = {}
	
	for i=1,GetNumPartyMembers() do
		local unit = "party"..i
		self.party[UnitName(unit)] = {
			canDrum = self.db.profile.tempUsers[UnitName(unit)] == true,
			isAlive = not UnitIsDeadOrGhost(unit),
			isOnline = UnitIsConnected(unit) == 1,
			}
	end

	self.party[UnitName("player")] = {
		canDrum = self.canDrum,
		isAlive = not UnitIsDeadOrGhost("player"),
		isOnline = true,
		}
	tinsert(self.party, tT)
	
--	table.sort(self.party)
--	self:SendCommMessage("DrumCoord", "get", "PARTY")
end

function DrumCoord:BAG_UPDATE()
	self:UpdateCanDrum()
end

function DrumCoord:COMBAT_LOG_EVENT_UNFILTERED(cleu, timestamp, cle, sGUID, sName, sFlags, dGUID, dName, dFlags, spellID, spellName, ...)	
	if (cle == "UNIT_DIED") then
		if not self.party[sName] then return end
		self.party[sName].isAlive = false
	elseif (cle == "SPELL_CAST_SUCCESS" and spellName=="Drums of Battle") then
		if not self.party[sName] then return end
		
		if not self.party[sName].canDrum then
			self.db.profile.tempUsers[sName] = true
			self.party[sName].canDrum = true
		end
		
		self:Announce(sName.."'s drums active!")
		
		if self.db.profile.prepWarning > 0 and self.db.profile.showDrumsDropping then
			self.warningTimer = self:ScheduleTimer(
				function()
					local source = sName
					self:Announce(source.."'s drums dropping in "..self.db.profile.prepWarning.."s...")
				end,
			30-self.db.profile.prepWarning)
		end
		if self.db.profile.showCooldownEnded then
			self.party[sName].cooldownTimer = self:ScheduleTimer(
				function()
					local source = sName
					self:Announce(source.."'s cooldown finished.")
				end,
			120)
		end
		
		self.nextTimer = self:ScheduleTimer(
			function()
				local source = sName
				local nD = self:GetNextAvailDrummer()
				if nD then
					self:Announce(source.."'s drums down! "..nD.."'s drums- Go now!")
				else
					self:Announce(source.."'s drums down! No drummer available.")
				end
			end
		, 30)
	end
end

function DrumCoord:RegisterWatchTimer()
	return nil
	-- if self.watchTimer then
		-- self:CancelTimer(self.watchTimer)
		-- self.watchTimer = nil
	-- end
	-- self.watchTimer = self:ScheduleRepeatingTimer(
		-- function()
			-- if self.inCombat and not GetPlayerBuffName("Drums of Battle") then
				-- local nD = self:GetNextAvailDrummer()
				-- if nD then
					-- self:Announce(nD)
				-- end
			-- end
		-- end, 10)
end

function DrumCoord:GetNextAvailDrummer()
	for k,v in pairs(self.party) do
		if (v.isOnline and v.canDrum and v.isAlive and not v.cooldownTimer) then
			return k--,v
		end
	end
	return nil
end

function DrumCoord:Debug(msg)
	self:Print("Debug: "..msg)
end

function DrumCoord:Announce(msg)
	if ((self.db.profile.showWhileSolo == false and GetNumPartyMembers() == 0) or
		(self.db.profile.showWhileUnusable == false and self.canDrum == false) or 
		(self.db.profile.showWhileNoCombat == false and self.inCombat == false)) then
		return
	end

	if self.db.profile.showInParty and GetNumPartyMembers() > 0 then	
		SendChatMessage(msg,"PARTY")
		self:SendCommMessage("DrumCoord",msg,"PARTY")
	end
	-- Here, a lot of doctoring of the color info for printing to chat window. Anyone suggest an improvement?
	if self.db.profile.showInChat then
		self:Print("|c"..self.db.profile.colorstr..msg.."|r")
	end
	
	if self.db.profile.showAsWarning then
		RaidNotice_AddMessage(RaidWarningFrame, msg, self.db.profile.messageColor) 
	end
end

function DrumCoord:ReceiveMessage(prefix, arg, distribution, sender)
--	self:Announce(prefix,sender,distribution,arg1)
	if not distribution == "PARTY" then 
		self:Print("Invalid comm <"..arg.."> recieved from "..sender.." via "..distribution)
		return
	end
	--self:Debug(sender..","..arg)
	if toboolean(arg)~=nil then
		self.party[sender].canDrum = toboolean(arg)
		self.party[sender].isOnline = true
	elseif arg=="get" then
		self:SendCommMessage("DrumCoord", tostring(self.canDrum), "PARTY")
	else
		RaidNotice_AddMessage(RaidWarningFrame, arg, self.db.profile.messageColor)
	end
end
		
function DrumCoord:IsShowInChat(info)
    return self.db.profile.showInChat
end

function DrumCoord:ToggleShowInChat(info, value)
    self.db.profile.showInChat = value
end

function DrumCoord:IsShowInParty(info)
    return self.db.profile.showInParty
end

function DrumCoord:ToggleShowInParty(info, value)
    self.db.profile.showInParty = value
end

function DrumCoord:IsShowOnCombat(info)
    return self.db.profile.showOnCombat
end

function DrumCoord:ToggleShowOnCombat(info, value)
    self.db.profile.showOnCombat = value
end

function DrumCoord:IsShowAsWarning(info)
    return self.db.profile.showAsWarning
end

function DrumCoord:ToggleShowAsWarning(info, value)
    self.db.profile.showAsWarning = value
end

function DrumCoord:IsShowWhileSolo(info)
    return self.db.profile.showWhileSolo
end

function DrumCoord:ToggleShowWhileSolo(info, value)
    self.db.profile.showWhileSolo = value
end

function DrumCoord:IsShowCooldownEnded(info)
    return self.db.profile.showCooldownEnded
end

function DrumCoord:ToggleShowCooldownEnded(info, value)
    self.db.profile.showCooldownEnded = value
end

function DrumCoord:IsShowDrumsDropping(info)
    return self.db.profile.showDrumsDropping
end

function DrumCoord:ToggleShowDrumsDropping(info, value)
    self.db.profile.showDrumsDropping = value
end

function DrumCoord:IsShowWhileUnusable(info)
    return self.db.profile.showWhileUnusable
end

function DrumCoord:ToggleShowWhileUnusable(info, value)
    self.db.profile.showWhileUnusable = value
end

function DrumCoord:IsShowWhileNoCombat(info)
    return self.db.profile.showWhileNoCombat
end

function DrumCoord:ToggleShowWhileNoCombat(info, value)
    self.db.profile.showWhileNoCombat = value
end

function DrumCoord:GetPrepWarning(info)
    return self.db.profile.prepWarning
end

function DrumCoord:SetPrepWarning(info, value)
    self.db.profile.prepWarning = value
end

function DrumCoord:GetMessageColor(info)
    return self.db.profile.messageColor.r, self.db.profile.messageColor.g, self.db.profile.messageColor.b, self.db.profile.messageColor.a
end

function DrumCoord:SetMessageColor(info, r, g, b, a)
    self.db.profile.messageColor.r = r
    self.db.profile.messageColor.g = g
    self.db.profile.messageColor.b = b
    self.db.profile.messageColor.a = a
	local temp = {}
	for key,value in pairs(self.db.profile.messageColor) do
		value = tostring(DEC_HEX(255*value))
		if strlen(value) == 1 then
			value = "0"..value
		end
		temp[key] = value
	end
	self.db.profile.colorstr = temp.a .. temp.r .. temp.g .. temp.b
end